'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var nock = require('nock');

var _require = require('chai'),
    expect = _require.expect;

var _require2 = require('node-localstorage'),
    LocalStorage = _require2.LocalStorage;

var _require3 = require('../src'),
    DataCapsule = _require3.DataCapsule,
    LocalStorageStrategy = _require3.LocalStorageStrategy,
    CachedStorageStrategy = _require3.CachedStorageStrategy,
    WixStorageStrategy = _require3.WixStorageStrategy;

describe('data-capsule', function () {
  describe('with localstorage strategy', function () {
    beforeEach(function () {
      global.localStorage = new LocalStorage('./scratch');
    });

    afterEach(function () {
      global.localStorage.clear();
    });

    it('should store and retrieve information', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      var capsule;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              capsule = new DataCapsule({ strategy: new LocalStorageStrategy() });
              _context.next = 3;
              return capsule.setItem('shahata', 123, { namespace: 'wix' });

            case 3:
              _context.t0 = expect;
              _context.next = 6;
              return capsule.getItem('shahata', { namespace: 'wix' });

            case 6:
              _context.t1 = _context.sent;
              (0, _context.t0)(_context.t1).to.equal(123);

            case 8:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('should cache items when setting items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
      var capsule;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              capsule = new DataCapsule({ strategy: new CachedStorageStrategy({
                  remoteStrategy: new WixStorageStrategy()
                }) });

              nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123 }).reply(200);
              _context2.next = 4;
              return capsule.setItem('shahata', 123, { namespace: 'wix' });

            case 4:
              _context2.t0 = expect;
              _context2.next = 7;
              return capsule.getItem('shahata', { namespace: 'wix' });

            case 7:
              _context2.t1 = _context2.sent;
              (0, _context2.t0)(_context2.t1).to.equal(123);

            case 9:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('should throw if non BaseStorage is passed', function () {
      expect(function () {
        return new DataCapsule({ strategy: {} });
      }).to.throw('must extend BaseStorage');
    });
  });
});