'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var nock = require('nock');
var sinon = require('sinon');

var _require = require('chai'),
    expect = _require.expect;

var _require2 = require('node-localstorage'),
    LocalStorage = _require2.LocalStorage;

var _require3 = require('../../src'),
    NOT_FOUND = _require3.NOT_FOUND,
    WixStorageStrategy = _require3.WixStorageStrategy,
    CachedStorageStrategy = _require3.CachedStorageStrategy,
    LocalStorageCachedCapsule = _require3.LocalStorageCachedCapsule;

describe('cached-storage-strategy', function () {
  beforeEach(function () {
    global.localStorage = new LocalStorage('./scratch');
  });

  afterEach(function () {
    global.localStorage.clear();
  });

  it('should cache items when setting items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123 }).reply(200);
            _context.next = 4;
            return capsule.setItem('shahata', 123, { namespace: 'wix' });

          case 4:
            _context.t0 = expect;
            _context.next = 7;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 7:
            _context.t1 = _context.sent;
            (0, _context.t0)(_context.t1).to.equal(123);

          case 9:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('should query server if item is not in cache', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(200, { shahata: 123 });
            _context2.t0 = expect;
            _context2.next = 5;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 5:
            _context2.t1 = _context2.sent;
            (0, _context2.t0)(_context2.t1).to.equal(123);

          case 7:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('should cache items when getting items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(200, { shahata: 123 });
            _context3.t0 = expect;
            _context3.next = 5;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 5:
            _context3.t1 = _context3.sent;
            (0, _context3.t0)(_context3.t1).to.equal(123);
            _context3.t2 = expect;
            _context3.next = 10;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 10:
            _context3.t3 = _context3.sent;
            (0, _context3.t2)(_context3.t3).to.equal(123);

          case 12:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it('should cache items only for one hour', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var clock, capsule;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            clock = sinon.useFakeTimers();
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(200, { shahata: 123 });
            _context4.t0 = expect;
            _context4.next = 6;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 6:
            _context4.t1 = _context4.sent;
            (0, _context4.t0)(_context4.t1).to.equal(123);

            clock.tick(3600000);
            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(200, { shahata: 123 });
            _context4.t2 = expect;
            _context4.next = 13;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 13:
            _context4.t3 = _context4.sent;
            (0, _context4.t2)(_context4.t3).to.equal(123);
            _context4.t4 = expect;
            _context4.next = 18;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 18:
            _context4.t5 = _context4.sent;
            (0, _context4.t4)(_context4.t5).to.equal(123);

            clock.restore();

          case 21:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  it('should cache items when getting all items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefs/wix').reply(200, { shahata: 123 });
            _context5.t0 = expect;
            _context5.next = 5;
            return capsule.getAllItems({ namespace: 'wix' });

          case 5:
            _context5.t1 = _context5.sent;
            _context5.t2 = { shahata: 123 };
            (0, _context5.t0)(_context5.t1).to.eql(_context5.t2);
            _context5.t3 = expect;
            _context5.next = 11;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 11:
            _context5.t4 = _context5.sent;
            (0, _context5.t3)(_context5.t4).to.equal(123);

          case 13:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  })));

  it('should cache also the knowledge about inexistence of items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(404);
            _context6.next = 4;
            return expect(capsule.getItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(NOT_FOUND);

          case 4:
            _context6.next = 6;
            return expect(capsule.getItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(NOT_FOUND);

          case 6:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  })));

  it('should cache also the knowledge about removal of items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            capsule = new LocalStorageCachedCapsule({ remoteStrategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/delete', { nameSpace: 'wix', key: 'shahata' }).reply(200);
            _context7.next = 4;
            return capsule.removeItem('shahata', { namespace: 'wix' });

          case 4:
            _context7.next = 6;
            return expect(capsule.getItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(NOT_FOUND);

          case 6:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  })));

  it('should throw if non BaseStorage is passed', function () {
    expect(function () {
      return new CachedStorageStrategy({});
    }).to.throw('must extend BaseStorage');
    expect(function () {
      return new CachedStorageStrategy(new WixStorageStrategy(), {});
    }).to.throw('must extend BaseStorage');
  });
});