'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var sinon = require('sinon');

var _require = require('chai'),
    expect = _require.expect;

var _require2 = require('node-localstorage'),
    LocalStorage = _require2.LocalStorage;

var _require3 = require('../../src'),
    NOT_FOUND = _require3.NOT_FOUND,
    LocalStorageCapsule = _require3.LocalStorageCapsule;

describe('local-storage-cleaner', function () {
  var oneKilobyte = new Array(1000).fill('a').join('');
  var clock = void 0;

  beforeEach(function () {
    clock = sinon.useFakeTimers(Date.now());
    global.localStorage = new LocalStorage('./limited', 4000); //only 4k in local storage
  });

  afterEach(function () {
    global.localStorage.clear();
    clock.restore();
  });

  it('should clean oldest records first', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'w' });

            clock.tick(1);
            _context.next = 4;
            return capsule.setItem('k1', oneKilobyte);

          case 4:
            clock.tick(1);
            _context.next = 7;
            return capsule.setItem('k2', oneKilobyte);

          case 7:
            clock.tick(1);
            _context.next = 10;
            return capsule.setItem('k3', oneKilobyte);

          case 10:
            clock.tick(1);
            _context.next = 13;
            return capsule.setItem('k4', oneKilobyte + oneKilobyte);

          case 13:
            _context.next = 15;
            return expect(capsule.getItem('k1')).to.be.rejectedWith(NOT_FOUND);

          case 15:
            _context.next = 17;
            return expect(capsule.getItem('k2')).to.be.rejectedWith(NOT_FOUND);

          case 17:
            _context.t0 = expect;
            _context.next = 20;
            return capsule.getItem('k3');

          case 20:
            _context.t1 = _context.sent;
            _context.t2 = oneKilobyte;
            (0, _context.t0)(_context.t1).to.equal(_context.t2);
            _context.t3 = expect;
            _context.next = 26;
            return capsule.getItem('k4');

          case 26:
            _context.t4 = _context.sent;
            _context.t5 = oneKilobyte + oneKilobyte;
            (0, _context.t3)(_context.t4).to.equal(_context.t5);

          case 29:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('should clean last accessed records first', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'w' });

            clock.tick(1);
            _context2.next = 4;
            return capsule.setItem('k1', oneKilobyte);

          case 4:
            clock.tick(1);
            _context2.next = 7;
            return capsule.setItem('k2', oneKilobyte);

          case 7:
            clock.tick(1);
            _context2.next = 10;
            return capsule.setItem('k3', oneKilobyte);

          case 10:
            clock.tick(1);
            _context2.next = 13;
            return capsule.getItem('k2');

          case 13:
            //access k2 so it is not removed first
            clock.tick(1);
            _context2.next = 16;
            return capsule.setItem('k4', oneKilobyte + oneKilobyte);

          case 16:
            _context2.next = 18;
            return expect(capsule.getItem('k1')).to.be.rejectedWith(NOT_FOUND);

          case 18:
            _context2.t0 = expect;
            _context2.next = 21;
            return capsule.getItem('k2');

          case 21:
            _context2.t1 = _context2.sent;
            _context2.t2 = oneKilobyte;
            (0, _context2.t0)(_context2.t1).to.equal(_context2.t2);
            _context2.next = 26;
            return expect(capsule.getItem('k3')).to.be.rejectedWith(NOT_FOUND);

          case 26:
            _context2.t3 = expect;
            _context2.next = 29;
            return capsule.getItem('k4');

          case 29:
            _context2.t4 = _context2.sent;
            _context2.t5 = oneKilobyte + oneKilobyte;
            (0, _context2.t3)(_context2.t4).to.equal(_context2.t5);

          case 32:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('should clean expired records first ', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'w' });

            clock.tick(1);
            _context3.next = 4;
            return capsule.setItem('k1', oneKilobyte);

          case 4:
            clock.tick(1);
            _context3.next = 7;
            return capsule.setItem('k2', oneKilobyte);

          case 7:
            clock.tick(1);
            _context3.next = 10;
            return capsule.setItem('k3', oneKilobyte, { expiration: 2 });

          case 10:
            clock.tick(2000); //k3 is expired so in addition we only remove k1 to free up 2k
            _context3.next = 13;
            return capsule.setItem('k4', oneKilobyte + oneKilobyte);

          case 13:
            _context3.next = 15;
            return expect(capsule.getItem('k1')).to.be.rejectedWith(NOT_FOUND);

          case 15:
            _context3.t0 = expect;
            _context3.next = 18;
            return capsule.getItem('k2');

          case 18:
            _context3.t1 = _context3.sent;
            _context3.t2 = oneKilobyte;
            (0, _context3.t0)(_context3.t1).to.equal(_context3.t2);
            _context3.next = 23;
            return expect(capsule.getItem('k3')).to.be.rejectedWith(NOT_FOUND);

          case 23:
            _context3.t3 = expect;
            _context3.next = 26;
            return capsule.getItem('k4');

          case 26:
            _context3.t4 = _context3.sent;
            _context3.t5 = oneKilobyte + oneKilobyte;
            (0, _context3.t3)(_context3.t4).to.equal(_context3.t5);

          case 29:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));
});