'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var sinon = require('sinon');

var _require = require('chai'),
    expect = _require.expect;

var _require2 = require('node-localstorage'),
    LocalStorage = _require2.LocalStorage;

var _require3 = require('../../src'),
    NOT_FOUND = _require3.NOT_FOUND,
    LocalStorageCapsule = _require3.LocalStorageCapsule;

describe('localstorage-strategy', function () {
  beforeEach(function () {
    global.localStorage = new LocalStorage('./scratch');
  });

  afterEach(function () {
    global.localStorage.clear();
  });

  it('should store and retrieve information', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            capsule = new LocalStorageCapsule();
            _context.next = 3;
            return capsule.setItem('shahata', 123, { namespace: 'wix' });

          case 3:
            _context.t0 = expect;
            _context.next = 6;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 6:
            _context.t1 = _context.sent;
            (0, _context.t0)(_context.t1).to.equal(123);

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('should throw if no namespace in setItem', function () {
    var capsule = new LocalStorageCapsule();
    expect(function () {
      return capsule.setItem('shahata', 123);
    }).to.throw('namespace is required');
  });

  it('should throw if no namespace in getItem', function () {
    var capsule = new LocalStorageCapsule();
    expect(function () {
      return capsule.getItem('shahata');
    }).to.throw('namespace is required');
  });

  it('should throw if namespace is not a string', function () {
    var capsule = new LocalStorageCapsule();
    expect(function () {
      return capsule.getItem('shahata', { namespace: 123 });
    }).to.throw('namespace should be a string');
  });

  it('should accept namespace in capsule constructor', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context2.next = 3;
            return capsule.setItem('shahata', 123);

          case 3:
            _context2.t0 = expect;
            _context2.next = 6;
            return capsule.getItem('shahata');

          case 6:
            _context2.t1 = _context2.sent;
            (0, _context2.t0)(_context2.t1).to.equal(123);

          case 8:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('should store namespaces in isolation', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            capsule = new LocalStorageCapsule();
            _context3.next = 3;
            return capsule.setItem('shahata', 123, { namespace: 'wix1' });

          case 3:
            _context3.next = 5;
            return capsule.setItem('shahata', 456, { namespace: 'wix2' });

          case 5:
            _context3.t0 = expect;
            _context3.next = 8;
            return capsule.getItem('shahata', { namespace: 'wix1' });

          case 8:
            _context3.t1 = _context3.sent;
            (0, _context3.t0)(_context3.t1).to.equal(123);
            _context3.t2 = expect;
            _context3.next = 13;
            return capsule.getItem('shahata', { namespace: 'wix2' });

          case 13:
            _context3.t3 = _context3.sent;
            (0, _context3.t2)(_context3.t3).to.equal(456);

          case 15:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it('should optionally pass a scope for additional isolation', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context4.next = 3;
            return capsule.setItem('shahata', 123, { scope: 'wix1' });

          case 3:
            _context4.next = 5;
            return capsule.setItem('shahata', 456, { scope: 'wix2' });

          case 5:
            _context4.t0 = expect;
            _context4.next = 8;
            return capsule.getItem('shahata', { scope: 'wix1' });

          case 8:
            _context4.t1 = _context4.sent;
            (0, _context4.t0)(_context4.t1).to.equal(123);
            _context4.t2 = expect;
            _context4.next = 13;
            return capsule.getItem('shahata', { scope: 'wix2' });

          case 13:
            _context4.t3 = _context4.sent;
            (0, _context4.t2)(_context4.t3).to.equal(456);

          case 15:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  it('should optionally pass a scope in constructor', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix', scope: 'scope' });
            _context5.next = 3;
            return capsule.setItem('shahata', 123);

          case 3:
            _context5.t0 = expect;
            _context5.next = 6;
            return capsule.getItem('shahata', { scope: 'scope' });

          case 6:
            _context5.t1 = _context5.sent;
            (0, _context5.t0)(_context5.t1).to.equal(123);

          case 8:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  })));

  it('should result in rejection if item is not found', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context6.next = 3;
            return expect(capsule.getItem('shahata')).to.be.rejectedWith(NOT_FOUND);

          case 3:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  })));

  it('should treat expiration as not found', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
    var clock, capsule;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            clock = sinon.useFakeTimers();
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context7.next = 4;
            return capsule.setItem('shahata', 123, { expiration: 2 });

          case 4:
            clock.tick(2000);
            _context7.next = 7;
            return expect(capsule.getItem('shahata')).to.be.rejectedWith(NOT_FOUND);

          case 7:
            clock.restore();

          case 8:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  })));

  it('should remove item', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context8.next = 3;
            return capsule.setItem('shahata', 123);

          case 3:
            _context8.t0 = expect;
            _context8.next = 6;
            return capsule.getItem('shahata');

          case 6:
            _context8.t1 = _context8.sent;
            (0, _context8.t0)(_context8.t1).to.equal(123);
            _context8.next = 10;
            return capsule.removeItem('shahata');

          case 10:
            _context8.next = 12;
            return expect(capsule.getItem('shahata')).to.be.rejectedWith(NOT_FOUND);

          case 12:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, undefined);
  })));

  it('should get all items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix', scope: 'scope' });
            _context9.next = 3;
            return capsule.setItem('shahata1', 123);

          case 3:
            _context9.next = 5;
            return capsule.setItem('shahata2', 456);

          case 5:
            _context9.t0 = expect;
            _context9.next = 8;
            return capsule.getAllItems();

          case 8:
            _context9.t1 = _context9.sent;
            _context9.t2 = {
              shahata1: 123,
              shahata2: 456
            };
            (0, _context9.t0)(_context9.t1).to.eql(_context9.t2);

          case 11:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, undefined);
  })));

  it('should get all items filtering other namespaces/scopes', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix', scope: 'scope' });
            _context10.next = 3;
            return capsule.setItem('shahata1', 123);

          case 3:
            _context10.next = 5;
            return capsule.setItem('shahata2', 456, { namespace: 'wix1' });

          case 5:
            _context10.next = 7;
            return capsule.setItem('shahata3', 789, { scope: 'scope1' });

          case 7:
            _context10.t0 = expect;
            _context10.next = 10;
            return capsule.getAllItems();

          case 10:
            _context10.t1 = _context10.sent;
            _context10.t2 = { shahata1: 123 };
            (0, _context10.t0)(_context10.t1).to.eql(_context10.t2);

          case 13:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, undefined);
  })));

  it('should get all items when user controls filtering', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            capsule = new LocalStorageCapsule();
            _context11.next = 3;
            return capsule.setItem('shahata', 1, { namespace: 'wix1' });

          case 3:
            _context11.next = 5;
            return capsule.setItem('shahata', 2, { namespace: 'wix1', scope: 'scope1' });

          case 5:
            _context11.next = 7;
            return capsule.setItem('shahata', 3, { namespace: 'wix1', scope: 'scope2' });

          case 7:
            _context11.next = 9;
            return capsule.setItem('shahata', 4, { namespace: 'wix2', scope: 'scope1' });

          case 9:
            _context11.t0 = expect;
            _context11.next = 12;
            return capsule.getAllItems({ namespace: 'wix1', scope: 'scope1' });

          case 12:
            _context11.t1 = _context11.sent;
            _context11.t2 = { shahata: 2 };
            (0, _context11.t0)(_context11.t1).to.eql(_context11.t2);
            _context11.t3 = expect;
            _context11.next = 18;
            return capsule.getAllItems({ namespace: 'wix1' });

          case 18:
            _context11.t4 = _context11.sent;
            _context11.t5 = { shahata: 1 };
            (0, _context11.t3)(_context11.t4).to.eql(_context11.t5);

          case 21:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, undefined);
  })));

  it('should get all items filtering expired', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
    var clock, capsule;
    return regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            clock = sinon.useFakeTimers();
            capsule = new LocalStorageCapsule({ namespace: 'wix', scope: 'scope' });
            _context12.next = 4;
            return capsule.setItem('shahata1', 123);

          case 4:
            _context12.next = 6;
            return capsule.setItem('shahata2', 456, { expiration: 2 });

          case 6:
            clock.tick(2000);
            _context12.t0 = expect;
            _context12.next = 10;
            return capsule.getAllItems();

          case 10:
            _context12.t1 = _context12.sent;
            _context12.t2 = { shahata1: 123 };
            (0, _context12.t0)(_context12.t1).to.eql(_context12.t2);

            clock.restore();

          case 14:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, undefined);
  })));

  it('should accept json scope', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            capsule = new LocalStorageCapsule({ namespace: 'wix' });
            _context13.next = 3;
            return capsule.setItem('shahata', 1, { scope: { userId: 123 } });

          case 3:
            _context13.t0 = expect;
            _context13.next = 6;
            return capsule.getAllItems({ scope: { userId: 123 } });

          case 6:
            _context13.t1 = _context13.sent;
            _context13.t2 = { shahata: 1 };
            (0, _context13.t0)(_context13.t1).to.eql(_context13.t2);

          case 9:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, undefined);
  })));
});