'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var nock = require('nock');

var _require = require('chai'),
    expect = _require.expect;

var _require2 = require('../../src'),
    SERVER_ERROR = _require2.SERVER_ERROR,
    NOT_FOUND = _require2.NOT_FOUND,
    DataCapsule = _require2.DataCapsule,
    WixStorageStrategy = _require2.WixStorageStrategy;

describe('wix-storage-strategy', function () {
  it('should set item', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123 }).reply(200);
            _context.next = 4;
            return capsule.setItem('shahata', 123, { namespace: 'wix' });

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('should set item with scope', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123, siteId: 456 }).reply(200);
            _context2.next = 4;
            return capsule.setItem('shahata', 123, { namespace: 'wix', scope: { siteId: 456 } });

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('should set item with expiration', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123, TTLInDays: 1 }).reply(200);
            _context3.next = 4;
            return capsule.setItem('shahata', 123, { namespace: 'wix', expiration: 1 });

          case 4:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it('should set item with expiration in days', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123, TTLInDays: 2 }).reply(200);
            _context4.next = 4;
            return capsule.setItem('shahata', 123, { namespace: 'wix', expiration: 60 * 60 * 48 });

          case 4:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  it('should set item with error handling', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/set', { nameSpace: 'wix', key: 'shahata', blob: 123 }).reply(500);
            _context5.next = 4;
            return expect(capsule.setItem('shahata', 123, { namespace: 'wix' })).to.be.rejectedWith(SERVER_ERROR);

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  })));

  it('should delete item', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/delete', { nameSpace: 'wix', key: 'shahata' }).reply(200);
            _context6.next = 4;
            return capsule.removeItem('shahata', { namespace: 'wix' });

          case 4:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  })));

  it('should delete item with scope', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/delete', { nameSpace: 'wix', key: 'shahata', siteId: 456 }).reply(200);
            _context7.next = 4;
            return capsule.removeItem('shahata', { namespace: 'wix', scope: { siteId: 456 } });

          case 4:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  })));

  it('should delete item with error handling', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').post('/_api/wix-user-preferences-webapp/delete', { nameSpace: 'wix', key: 'shahata' }).reply(500);
            _context8.next = 4;
            return expect(capsule.removeItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(SERVER_ERROR);

          case 4:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, undefined);
  })));

  it('should get item', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(200, { shahata: 123 });
            _context9.t0 = expect;
            _context9.next = 5;
            return capsule.getItem('shahata', { namespace: 'wix' });

          case 5:
            _context9.t1 = _context9.sent;
            (0, _context9.t0)(_context9.t1).to.equal(123);

          case 7:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, undefined);
  })));

  it('should get item with scope', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForSite/wix/456/shahata').reply(200, { shahata: 123 });
            _context10.t0 = expect;
            _context10.next = 5;
            return capsule.getItem('shahata', { namespace: 'wix', scope: { siteId: 456 } });

          case 5:
            _context10.t1 = _context10.sent;
            (0, _context10.t0)(_context10.t1).to.equal(123);

          case 7:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, undefined);
  })));

  it('should get item with error handling', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(500);
            _context11.next = 4;
            return expect(capsule.getItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(SERVER_ERROR);

          case 4:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, undefined);
  })));

  it('should get item with 404 handling', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefForKey/wix/shahata').reply(404);
            _context12.next = 4;
            return expect(capsule.getItem('shahata', { namespace: 'wix' })).to.be.rejectedWith(NOT_FOUND);

          case 4:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, undefined);
  })));

  it('should get items', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefs/wix').reply(200, { shahata: 123 });
            _context13.t0 = expect;
            _context13.next = 5;
            return capsule.getAllItems({ namespace: 'wix' });

          case 5:
            _context13.t1 = _context13.sent;
            _context13.t2 = { shahata: 123 };
            (0, _context13.t0)(_context13.t1).to.eql(_context13.t2);

          case 8:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, undefined);
  })));

  it('should get items with scope', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefsForSite/wix/456').reply(200, { shahata: 123 });
            _context14.t0 = expect;
            _context14.next = 5;
            return capsule.getAllItems({ namespace: 'wix', scope: { siteId: 456 } });

          case 5:
            _context14.t1 = _context14.sent;
            _context14.t2 = { shahata: 123 };
            (0, _context14.t0)(_context14.t1).to.eql(_context14.t2);

          case 8:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, undefined);
  })));

  it('should get items with error handling', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee15() {
    var capsule;
    return regeneratorRuntime.wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            capsule = new DataCapsule({ strategy: new WixStorageStrategy() });

            nock('http://localhost').get('/_api/wix-user-preferences-webapp/getVolatilePrefs/wix').reply(500);
            _context15.next = 4;
            return expect(capsule.getAllItems({ namespace: 'wix' })).to.be.rejectedWith(SERVER_ERROR);

          case 4:
          case 'end':
            return _context15.stop();
        }
      }
    }, _callee15, undefined);
  })));
});